/*
 * Copyright (C) 2003 Ross Burton <ross@burtonini.com>
 * 
 * Some code from DrWright:
 *   Copyright (C) 2002 Richard Hult <richard@imendio.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAS_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>
#include "rb-black-point-window.h"

struct _RbBlackPointWindowPriv {
};

/* Signals */
enum {
	DONE,
	LAST_SIGNAL
};

static void rb_black_point_window_class_init (RbBlackPointWindowClass *klass);
static void rb_black_point_window_init (RbBlackPointWindow *window);
static void rb_black_point_window_finalize (GObject *object);

static GObjectClass *parent_class;
static guint signals[LAST_SIGNAL];


GType
rb_black_point_window_get_type (void)
{
  static GType object_type = 0;
  if (!object_type) {
    static const GTypeInfo object_info = {
      sizeof (RbBlackPointWindowClass),
      NULL,		/* base_init */
      NULL,		/* base_finalize */
      (GClassInitFunc) rb_black_point_window_class_init,
      NULL,		/* class_finalize */
      NULL,		/* class_data */
      sizeof (RbBlackPointWindow),
      0,              /* n_preallocs */
      (GInstanceInitFunc) rb_black_point_window_init,
    };
    object_type = g_type_register_static (GTK_TYPE_WINDOW,
                                          "RbBlackPointWindow", 
                                          &object_info,
                                          0);
  }
  return object_type;
}

static void
rb_black_point_window_class_init (RbBlackPointWindowClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  
  parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
  
  object_class->finalize = rb_black_point_window_finalize;
  
  signals[DONE] = g_signal_new ("done",
                                G_TYPE_FROM_CLASS (klass),
                                G_SIGNAL_RUN_LAST,
                                0,
                                NULL, NULL,
                                g_cclosure_marshal_VOID__VOID,
                                G_TYPE_NONE, 0);
}

static void
rb_black_point_window_init (RbBlackPointWindow *window)
{
  RbBlackPointWindowPriv *priv;
  PangoContext *context;
  PangoLayout *layout;
  int width, height;
  int i, block_width;
  GdkPixmap *pixmap;
  GdkColormap *colourmap;
  GdkGC *gc;
  GdkColor *black, *white;
  GdkColor colours[] = {
    {0, 0.006 * 65535, 0.006 * 65535, 0.006 * 65535},
    {0, 0.008 * 65535, 0.008 * 65535, 0.008 * 65535},
    {0, 0.010 * 65535, 0.010 * 65535, 0.010 * 65535},
    {0, 0.012 * 65535, 0.012 * 65535, 0.012 * 65535},
    {0, 0.014 * 65535, 0.014 * 65535, 0.014 * 65535},
    {0, 0.020 * 65535, 0.020 * 65535, 0.020 * 65535}
  };

  priv = g_new0 (RbBlackPointWindowPriv, 1);
  window->priv = priv;
 
  width = gdk_screen_width ();
  height = gdk_screen_height ();
  block_width = width / 6;

  gtk_window_set_default_size (GTK_WINDOW (window),
                               width, height);
  gtk_window_fullscreen (GTK_WINDOW (window));

  gtk_widget_set_app_paintable (GTK_WIDGET (window), TRUE);
  gtk_widget_realize (GTK_WIDGET (window));
  gtk_window_stick (GTK_WINDOW (window));

  pixmap = gdk_pixmap_new (GTK_WIDGET (window)->window,
                           width, height, -1);
  colourmap = gdk_drawable_get_colormap (GDK_DRAWABLE (pixmap));

  white = &(GTK_WIDGET (window)->style->white);
  black = &(GTK_WIDGET (window)->style->black);

  gc = gdk_gc_new (pixmap);
  gdk_gc_set_rgb_fg_color (gc, black);
  gdk_draw_rectangle (pixmap, gc, TRUE, 0, 0, width, height);

  context = gtk_widget_get_pango_context (GTK_WIDGET (window));

  for (i = 0; i < 6; ++i) { /* TODO: remove hard coded range */
    int layout_height;
    char *str;
    gdk_rgb_find_color (colourmap, &colours[i]);
    gdk_gc_set_rgb_fg_color (gc, &colours[i]);
    gdk_draw_rectangle (pixmap, gc, TRUE, i*block_width + 10, 100, block_width-20, 200);

    layout = pango_layout_new (context);
    str = g_strdup_printf ("<span foreground=\"white\">%.2f%%</span>", colours[i].red / 65536.0 * 100);
    pango_layout_set_markup (layout, str, -1);
    g_free (str);
    pango_layout_set_alignment (layout, PANGO_ALIGN_CENTER);
    pango_layout_set_width (layout, block_width * PANGO_SCALE);
    pango_layout_get_pixel_size (layout, NULL, &layout_height);
    gdk_draw_layout (pixmap, gc, i*block_width, 300 + layout_height, layout);
    g_object_unref (G_OBJECT (layout));
  }

  layout = pango_layout_new (context);
  pango_layout_set_markup (layout, "<span size=\"large\" foreground=\"white\">Adjust the brightness until you can see the four squares on the right.\n\nPress any key to close this window.</span>", -1);
  pango_layout_set_alignment (layout, PANGO_ALIGN_CENTER);
  pango_layout_set_width (layout, width * PANGO_SCALE);
  gdk_draw_layout (pixmap, gc, 0, 400, layout);
  g_object_unref (G_OBJECT (layout));

  gdk_window_set_back_pixmap (GTK_WIDGET (window)->window, pixmap, FALSE);

  {
    GdkCursor *cursor;
    unsigned char d[] = {0};
    GdkPixmap *pixmap;
    pixmap = gdk_bitmap_create_from_data (NULL, d, 1, 1);
    cursor = gdk_cursor_new_from_pixmap (pixmap, pixmap, black, black, 0, 0);
    gdk_window_set_cursor (GTK_WIDGET (window)->window, cursor);
    gdk_cursor_unref (cursor);
    /*gdk_display_sync (gdk_drawable_get_display (GTK_WIDGET (window)->window)); TODO: why do this? */
  }

  gtk_widget_add_events (GTK_WIDGET (window), GDK_KEY_PRESS_MASK | GDK_BUTTON_PRESS_MASK);
}

static void
rb_black_point_window_finalize (GObject *object)
{
  RbBlackPointWindow *window = RB_BLACK_POINT_WINDOW (object);
  RbBlackPointWindowPriv *priv;  

  priv = window->priv;
  g_free (priv);
  window->priv = NULL;

  // TODO: gdk_colormap_free_colors (colourmap, colours, XX;
  
  if (G_OBJECT_CLASS (parent_class)->finalize) {
    (*G_OBJECT_CLASS (parent_class)->finalize) (object);
  }
}

GtkWidget *
rb_black_point_window_new (void)
{
  return g_object_new (RB_TYPE_BLACK_POINT_WINDOW, NULL);
}
